#include <iostream>
#include <string>
#include "ipglasma.hpp"
#include <cmath>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <omp.h>

using namespace std;

int printDipoleAmplitudes(string fname, IPGlasma ipglasma)
{
    //ofstream output0;
    //Initialize params
    double q1[2] = {0,0};
    double q2[2] = {0,0};
    double r = 0;
    double dipoleAmp = 0;
    //TODO: set number of different rotations for dipole!
    int N = 100;

    cout << fixed << setprecision(8);
    
    cout << "# Dipole amplitudes for b=0"<< endl;
    cout << "# " << N << " rotations " << endl;
    cout << "# Filename " << fname << endl;
    cout << "# -------------------------"<< endl;
    cout << "# r, D_p(r,0)" << endl;

    // loop to calculate dipole amplitude at radii r=j*10^i
    for (int i=-4; i < 2; i += 1){
        cout <<""<< endl;

        // use index j to get radius
        for (double j=1; j < 10; j += 1){
            r = j*pow(10, i);
            // go through different orientations of the dipole via rotations
            for (double k=0; k < N; k += 1){
            // inside cos/sin is k*1/N*2*3.141592
                q1[0] = 0.5*r*cos(k*1./N*2*3.141592);
                q1[1] = 0.5*r*sin(k*1./N*2*3.141592);
                q2[0] = -q1[0];
                q2[1] = -q1[1];
    
                dipoleAmp += ipglasma.Amplitude(0, q1, q2);
            }

            dipoleAmp = dipoleAmp/N;
            cout << r << ", " << dipoleAmp << endl;
            dipoleAmp = 0;
        }
    }

    return 0; 
}


int main(int argc, char* argv[])
{
    if (argc < 3 || argc > 4){
        cerr << "Usage: " << argv[0] << " filename 1(integrate)/0(print amplitudes) [task_id]" << endl;
    }
    
    string fname = argv[1];
    IPGlasma ipglasma(fname, 0, BINARY);

    if (!argv[2]){ 
        printDipoleAmplitudes(fname, ipglasma);
    }

    int task_id = 0;
    if (argc > 3){ 
        task_id= atoi(argv[3]);
        cout << "#Integrating array job #" << task_id << endl;
        ipglasma.SumAmplitudes(task_id);   
    }

    
    else{ 
        double sum = ipglasma.SumAmplitudes();
        // Save results
        //std::filesystem::create_directory("output_"+fname);
        ofstream outfile("da_sum_ +" + fname + ".txt");
        outfile << sum << endl;
	outfile.close();
    }

    return 0;
}   
